'www.PredictiveEngineering.com
'All Rights Reserved, 2013
'Predictive Engineering Assumes No Responsibility For Results Obtained From API
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'This API was originally coded for FEMAP v10.2
'Written by Adrian Jensen
'This API clears all entities (besides solids) from a group
'It then adds all related entities (surfaces, curves, points, elements, nodes, connection regions) to that group

Sub Main
	Dim App As femap.model
	Set App = feFemap()

'create a set for geometric points
	Dim ptSet As femap.Set
	Set ptSet = App.feSet

'create a set for geometric curves
	Dim cuSet As femap.Set
	Set cuSet = App.feSet

'create a set for geometric surfaces
	Dim suSet As femap.Set
	Set suSet = App.feSet

'create a set for geometric solids
	Dim sldSet As femap.Set
	Set sldSet = App.feSet

'create a set for nodes
	Dim ndSet As femap.Set
	Set ndSet = App.feSet

'create a set for elements
	Dim elSet As femap.Set
	Set elSet = App.feSet

'create a set for regions
	Dim rgSet As femap.Set
	Set rgSet = App.feSet

'create a group object
	Dim feGroup As femap.Group
	Set feGroup = App.feGroup

'select the ID of the active group
	grID = App.Info_ActiveID( FT_GROUP )

'get the active group from the Femap database with the group object
	rc = feGroup.Get( grID )

'add all solids in the active group to the solid set
	rc = sldSet.AddGroup( FT_SOLID, grID )

'add all surfaces on solids (in the solid set) to the surface set
'add all curves on surfaces (in the surface set) to the curve set
'add all points on curves (in the curve set) to the point set
	rc = suSet.AddSetRule( sldSet.ID, FGD_SURFACE_ONSOLID )
	rc = cuSet.AddSetRule( suSet.ID, FGD_CURVE_ONSURFACE )
	rc = ptSet.AddSetRule( cuSet.ID, FGD_POINT_ONCURVE )

'add all elements in solids (in the solid set) to the element set
'add all elements on surfaces (in the surface set) to the element set
'add all nodes on elements (in the element set) to the node set
	rc = elSet.AddSetRule( sldSet.ID, FGD_ELEM_ATSOLID )
	rc = elSet.AddSetRule( suSet.ID, FGD_ELEM_ATSURFACE )
	rc = ndSet.AddSetRule( elSet.ID, FGD_NODE_ONELEM )

'add all elements in solids (in the solid set) to the element set
'add all elements on surfaces (in the surface set) to the element set
'add all nodes on elements (in the element set) to the node set
	rc = rgSet.AddSetRule( suSet.ID, FGD_CONTACT_BYSURFACE)
	rc = rgSet.AddSetRule( cuSet.ID, FGD_CONTACT_BYCURVE)
	rc = rgSet.AddSetRule( elSet.ID, FGD_CONTACT_BYELEM)
	rc = rgSet.AddSetRule( ndSet.ID, FGD_CONTACT_BYNODE)

'Clear out all previous entities
	rc = feGroup.RangeDeleteAll(-1)
'	rc = feGroup.RangeDeleteAll(FGD_POINT_ID)
'	rc = feGroup.RangeDeleteAll(FGD_CURVE_ID)
'	rc = feGroup.RangeDeleteAll(FGD_SURFACE_ID)
'	rc = feGroup.RangeDeleteAll(FGD_NODE_ID)
'	rc = feGroup.RangeDeleteAll(FGD_ELEM_ID)

'add the entities in all sets to the group
	rc = feGroup.SetAdd(FT_SOLID, sldSet.ID)
	rc = feGroup.SetAdd( FT_SURFACE, suSet.ID )
	rc = feGroup.SetAdd( FT_CURVE, cuSet.ID )
	rc = feGroup.SetAdd( FT_POINT, ptSet.ID )

	rc = feGroup.SetAdd( FT_NODE, ndSet.ID )
	rc = feGroup.SetAdd( FT_ELEM, elSet.ID )

	rc = feGroup.SetAdd(FT_CONTACT, rgSet.ID)

'put the group back into the Femap database
	rc = feGroup.Put( grID )

	rc = App.feViewRegenerate( 0 )

End Sub
